@@ -15,7 +15,7 @@ sub parse_arg {
     my ($self, $path, @arg) = @_;
 
     # Allow "svk mi uri://... //depot" to mean "svk mi //depot uri://"
-    if (@arg and $path =~ m{\w://} and $arg[0] !~ m{\w://}) {
+    if (@arg and $path =~ m{^\w+:}) {
 	($arg[0], $path) = ($path, $arg[0]);
     }
 
@@ -55,6 +55,7 @@ sub run {
 			      repospath => $target->{repospath},
 			      repos => $target->{repos},
 			      options => \@options,
+			      config => $self->{svnconfig},
 			      pool => SVN::Pool->new, auth => $self->auth,
 			      # XXX: remove in next svn::mirror release
 			      target => $target->{repospath},
@@ -84,8 +85,6 @@ SVK::Command::Mirror - Initialize a mirrored depotpath
  mirror --list
  mirror --upgrade /DEPOT/
 
-=head1 DESCRIPTION
-
 =head1 OPTIONS
 
  --list:               List mirrored path
