@@ -76,9 +76,8 @@ sub run {
 		( cb_flush => sub {
 		      my ($path, $status) = @_;
 		      my $st = $status->[0];
-		      my $dpath = $path ?
-			  File::Spec->catfile ($target->{path}, $path) : $target->{path};
-		      my $copath = $path ? "$target->{copath}/$path" : $target->{copath};
+		      my $dpath = $path ? "$target->{path}/$path" : $target->{path};
+		      my $copath = $target->copath ($path);
 		      if ($st eq 'M' || $st eq 'D' || $st eq '!' || $st eq 'R') {
 			  $revert->($dpath, $copath);
 		      }
@@ -91,7 +90,7 @@ sub run {
     }
     else {
 	if ($target->{targets}) {
-	    &$revert_item ("$target->{path}/$_", "$target->{copath}/$_")
+	    &$revert_item ("$target->{path}/$_", $target->copath ($_))
 		for @{$target->{targets}};
 	}
 	else {
