@@ -21,6 +21,7 @@ use SVN::Ra;
 my $svn_mirror = eval 'require SVN::Mirror; 1' ? 1 : 0;
 sub svn_mirror () {
     no warnings 'redefine';
+    local $@;
     my $svn_mirror = eval { require SVN::Mirror; 1 };
     *svn_mirror = $svn_mirror ? sub () { 1 } : sub () { 0 };
     return $svn_mirror;
@@ -190,11 +191,12 @@ sub abs_path {
 
 sub mimetype {
     no warnings 'redefine';
+    local $@;
     my $mimetype = eval {
-        require File::MimeInfo::Magic; 
+        require File::MimeInfo::Magic;
         \&File::MimeInfo::Magic::mimetype;
     };
-    *mimetype = $mimetype || sub { undef };
+    *mimetype = $mimetype ||= sub { undef };
     goto &$mimetype;
 }
 
